@@ -10,21 +10,32 @@ intermittentParametersSetter <- function(intermittent="n",...){
         obsNonzero <- sum(ot);
         # 1 parameter for estimating initial probability
         nParamIntermittent <- 1;
-#         if(intermittent=="c"){
-#             # In Croston we also need to estimate smoothing parameter and variance
-#            nParamIntermittent <- nParamIntermittent + 2;
-#         }
-#         else if(any(intermittent==c("t","a"))){
-#             # In TSB we also need to estimate smoothing parameter and two parameters of distribution...
-#            nParamIntermittent <- nParamIntermittent + 3;
-#         }
+        if(intermittent=="t"){
+            # In TSB we only need to estimate smoothing parameter - we do not
+            # estimate any parameters of the Beta distribution.
+            nParamIntermittent <- nParamIntermittent + 1;
+        }
+        else if(any(intermittent==c("c","a"))){
+            # In Croston we also need to estimate smoothing parameter and variance
+            nParamIntermittent <- nParamIntermittent + 2;
+        }
         yot <- matrix(y[y!=0],obsNonzero,1);
         if(!imodelProvided){
             pt <- matrix(mean(ot),obsInsample,1);
             pt.for <- matrix(1,h,1);
         }
         else{
-            pt <- matrix(imodel$fitted,obsInsample,1);
+            if(length(imodel$fitted)>obsInsample){
+                pt <- matrix(imodel$fitted[1:obsInsample],obsInsample,1);
+            }
+            else if(length(imodel$fitted)<obsInsample){
+                pt <- matrix(c(imodel$fitted,
+                               rep(imodel$fitted[length(imodel$fitted)],obsInsample-length(imodel$fitted))),
+                             obsInsample,1);
+            }
+            else{
+                pt <- matrix(imodel$fitted,obsInsample,1);
+            }
             pt.for <- matrix(imodel$forecast,h,1);
             iprob <- c(pt,pt.for);
         }
@@ -33,11 +44,13 @@ intermittentParametersSetter <- function(intermittent="n",...){
         obsNonzero <- obsInsample;
     }
 
+    if(intermittent!="n"){
 # If number of observations is low, set intermittency to "none"
-    if(obsNonzero < 5){
-        warning(paste0("Not enough non-zero observations for intermittent state-space model. We need at least 5.\n",
-                       "Changing intermittent to 'n'."),call.=FALSE);
-        intermittent <- "n";
+        if(obsNonzero < 5){
+            warning(paste0("Not enough non-zero observations for intermittent state-space model. We need at least 5.\n",
+                           "Changing intermittent to 'n'."),call.=FALSE);
+            intermittent <- "n";
+        }
     }
 
     if(intermittent=="n"){
@@ -73,17 +86,20 @@ intermittentMaker <- function(intermittent="n",...){
             imodel <- iss(y, model=intermittentModel, intermittent=intermittent, h=h,
                           persistence=imodel$persistence, initial=imodel$initial);
         }
+        nParamIntermittent <- imodel$nParam;
         pt[,] <- imodel$fitted;
         pt.for <- imodel$forecast;
         iprob <- pt.for[1];
     }
     else{
         imodel <- NULL;
+        nParamIntermittent <- 0;
     }
 
     assign("pt",pt,ParentEnvironment);
     assign("pt.for",pt.for,ParentEnvironment);
     assign("iprob",iprob,ParentEnvironment);
+    assign("nParamIntermittent",nParamIntermittent,ParentEnvironment);
     assign("imodel",imodel,ParentEnvironment);
 }
 
@@ -108,6 +124,7 @@ intermittentMaker <- function(intermittent="n",...){
 #' Teunter et al., 2011 method., \code{"sba"} - Syntetos-Boylan Approximation
 #' for Croston's method (bias correction) discussed in Syntetos and Boylan,
 #' 2005.
+#' @param ic Information criteria to use in case of model selection.
 #' @param h Forecast horizon.
 #' @param holdout If \code{TRUE}, holdout sample of size \code{h} is taken from
 #' the end of the data.
@@ -115,6 +132,8 @@ intermittentMaker <- function(intermittent="n",...){
 #' be either \code{"ANN"} or \code{"MNN"}.
 #' @param persistence Persistence vector. If \code{NULL}, then it is estimated.
 #' @param initial Initial vector. If \code{NULL}, then it is estimated.
+#' @param xreg Vector of matrix of exogenous variables, explaining some parts
+#' of occurrence variable (probability).
 #' @return The object of class "iss" is returned. It contains following list of
 #' values:
 #'
@@ -143,8 +162,8 @@ intermittentMaker <- function(intermittent="n",...){
 #'     iss(y, intermittent="c", persistence=0.1)
 #'
 #' @export iss
-iss <- function(data, intermittent=c("none","fixed","croston","tsb","sba"),
-                h=10, holdout=FALSE, model=NULL, persistence=NULL, initial=NULL){
+iss <- function(data, intermittent=c("none","fixed","croston","tsb","sba"),ic=c("AICc","AIC","BIC"),
+                h=10, holdout=FALSE, model=NULL, persistence=NULL, initial=NULL, xreg=NULL){
 # Function estimates and returns mean and variance of probability for intermittent State-Space model based on the chosen method
     intermittent <- substring(intermittent[1],1,1);
     if(all(intermittent!=c("n","f","c","t","s"))){
@@ -239,13 +258,13 @@ iss <- function(data, intermittent=c("none","fixed","croston","tsb","sba"),
         zeroes <- diff(zeroes);
 # Number of intervals in Croston
         iyt <- matrix(zeroes,length(zeroes),1);
-        newh <- which(y!=0)
+        newh <- which(y!=0);
         newh <- newh[length(newh)];
-        newh <- obsInsample - newh + h
+        newh <- obsInsample - newh + h;
         crostonModel <- es(iyt,model=model,silent=TRUE,h=newh,
-                           persistence=persistence,initial=initial);
+                           persistence=persistence,initial=initial,
+                           ic=ic,xreg=xreg);
 
-        zeroes[length(zeroes)] <- zeroes[length(zeroes)];
         pt <- rep((crostonModel$fitted),zeroes);
         tailNumber <- obsInsample - length(pt);
         if(tailNumber>0){
@@ -273,95 +292,46 @@ iss <- function(data, intermittent=c("none","fixed","croston","tsb","sba"),
     }
 #### TSB method ####
     else if(intermittent=="t"){
-        ivt <- matrix(rep(iprob,obsInsample+1),obsInsample+1,1);
+        if(is.null(model)){
+            model <- "YYY";
+        }
+        if(is.null(initial)){
+            initial <- "o";
+        }
+
         iyt <- matrix(ot,obsInsample,1);
-        modellags <- matw <- matF <- matrix(1,1,1);
-
-        if(!is.null(model)){
-            if(model!="MNN"){
-                warning("Sorry, but currently TSB can only use ETS(M,N,N) model.", call.=FALSE);
-                model <- "MNN";
-                Etype <- "M";
-                Ttype <- "N";
-                Stype <- "N";
-            }
-        }
-
-        if(!is.null(persistence)){
-            if(length(persistence)!=1){
-                warning("Only one smoothing parameter is currently supported for TSB. Using the first value.",
-                        call.=FALSE);
-                persistence <- persistence[1];
-            }
-            persistenceEstimate <- FALSE;
-            vecg <- matrix(persistence,1,1);
-            C <- c(ivt[1]);
-            CLower <- c(0);
-            CUpper <- c(1);
-        }
-        else{
-            persistenceEstimate <- TRUE;
-            vecg <- matrix(0.1,1,1);
-            C <- c(ivt[1],vecg[1]);
-            CLower <- c(0,0);
-            CUpper <- c(1,1);
-        }
-
-        errors <- matrix(NA,obsInsample,1);
-        iyt.fit <- matrix(NA,obsInsample,1);
-
-#### CF for initial and persistence ####
-        CF <- function(C){
-            ivt[1,] <- C[1];
-            if(persistenceEstimate){
-                vecg[,] <- C[2];
-            }
-
-            fitting <- fitterwrap(ivt, matF, matw, iy_kappa, vecg,
-                                  modellags, Etype, Ttype, Stype, "o",
-                                  matrix(0,obsInsample,1), matrix(0,obsInsample+1,1),
-                                  matrix(1,1,1), matrix(1,1,1), matrix(1,obsInsample,1));
-
-            iyt.fit <- fitting$yfit;
-
-            CF.res <- -(sum(log(iyt.fit[ot==1])) + sum(log((1-iyt.fit[ot==0]))));
-            return(CF.res);
-        }
+        iyt <- ts(iyt,frequency=frequency(data));
 
         kappa <- 1E-5;
         iy_kappa <- iyt*(1 - 2*kappa) + kappa;
 
-        # Another run, now to define persistence and initial
-        res <- nloptr(C, CF, lb=CLower, ub=CUpper,
-                      opts=list("algorithm"="NLOPT_LN_BOBYQA", "xtol_rel"=1e-6, "maxeval"=100));
-
-        ivt[1,] <- res$solution[1];
-        if(persistenceEstimate){
-            vecg[,] <- res$solution[2];
-        }
-
-        logLik <- structure(-res$objective,df=3,class="logLik");
-
-        iy_kappa <- iyt*(1 - 2*kappa) + kappa;
-        fitting <- fitterwrap(ivt, matF, matw, iy_kappa, vecg,
-                              modellags, Etype, Ttype, Stype, "o",
-                              matrix(0,obsInsample,1), matrix(0,obsInsample+1,1),
-                              matrix(1,1,1), matrix(1,1,1), matrix(1,obsInsample,1));
-
-        ivt <- ts(fitting$matvt,start=(time(y)[1] - deltat(y)),frequency=frequency(y));
-        iyt.fit <- ts(fitting$yfit,start=start(y),frequency=frequency(y));
-        errors <- ts(fitting$errors,start=start(y),frequency=frequency(y));
-        iyt.for <- ts(rep(iyt.fit[obsInsample],h),
-                     start=time(y)[obsInsample]+deltat(y),frequency=frequency(y));
+        tsbModel <- es(iy_kappa,model,persistence=persistence,initial=initial,
+                       ic=ic,silent=TRUE,h=h,cfType="TSB",xreg=xreg);
 
         # Correction so we can return from those iy_kappa values
-        iyt.fit <- (iyt.fit - kappa) / (1 - 2*kappa);
-        iyt.for <- (iyt.for - kappa) / (1 - 2*kappa);
+        tsbModel$fitted <- (tsbModel$fitted - kappa) / (1 - 2*kappa);
+        tsbModel$forecast <- (tsbModel$forecast - kappa) / (1 - 2*kappa);
 
-        output <- list(model=model, fitted=iyt.fit, forecast=iyt.for, states=ivt,
-                       variance=iyt.for*(1-iyt.for), logLik=logLik, nParam=3,
-                       residuals=errors, actuals=otAll,
-                       persistence=vecg, initial=ivt[1,]);
+        # If bt>1, then at = 0 and pt = bt / (at + bt) = 1
+        if(any(tsbModel$fitted>1)){
+            tsbModel$fitted[tsbModel$fitted>1] <- 1;
+        }
+        if(any(tsbModel$forecast>1)){
+            tsbModel$forecast[tsbModel$forecast>1] <- 1;
+        }
+
+        # If at>1, then bt = 0 and pt = bt / (at + bt) = 0
+        if(any(tsbModel$fitted<0)){
+            tsbModel$fitted[tsbModel$fitted<0] <- 0;
+        }
+        if(any(tsbModel$forecast<0)){
+            tsbModel$forecast[tsbModel$forecast<0] <- 0;
+        }
+
+        output <- list(model=model, fitted=tsbModel$fitted, forecast=tsbModel$forecast, states=tsbModel$states,
+                       variance=tsbModel$forecast*(1-tsbModel$forecast), logLik=logLik(tsbModel), nParam=nParam(tsbModel)-1,
+                       residuals=tsbModel$residuals, actuals=otAll,
+                       persistence=tsbModel$persistence, initial=tsbModel$initial);
     }
 #### None ####
     else{
